<?php
namespace DEPOClient;


use DEPOClient\Entities\Label;
use DEPOClient\Entities\Place;
use DEPOClient\Entities\Package;
use DEPOClient\Exceptions\DEPOException;

interface ClientInterface {

	/**
	 * Returns array of all available Places.
	 * Throws exception on failure.
	 * @return Place[] (with id as key)
	 * @throws DEPOException
	 */
	public function getPlaces();

	/**
	 * Returns Place identified by this id.
	 * Throws exception on failure.
	 * @param $id
	 * @return mixed
	 * @throws DEPOException
	 */
	public function getPlace($id);

	/**
	 * Returns Package identified by this number.
	 * Throws exception on failure.
	 * @param $number
	 * @return Package
	 * @throws DEPOException
	 */
	public function getPackage($number);

	/**
	 * Returns array of all Packages.
	 * Throws exception on failure.
	 * @return Package[]
	 * @return mixed
	 */
	public function getPackages();

	/**
	 * Sends a Package. Sets its number and price.
	 * Throws exception on failure.
	 * @param Package $package
	 * @return Package
	 * @throws DEPOException
	 */
	public function sendPackage(Package $package);

	/**
	 * Cancels a Package (using its number as identifier)
	 * Throws exception on failure
	 * @param $number
	 * @throws DEPOException
	 */
	public function cancelPackage($number);

	/**
	 * Generates Label pdf.
	 * The pdf is stored on disk. Full path to file
	 * is assigned to Label::pdf_filename.
	 * @param Label $label
	 * @return Label
	 * @throws DEPOException
	 */
	public function generateLabel(Label $label);

}