<?php

require_once 'client/bootstrap.php';
require_once 'classes/CartDepoExt.php';
require_once 'classes/OrderDepoExt.php';
require_once 'classes/OrderDepoExtHistory.php';

require_once 'controllers/admin/AdminDepoCarrierController.php';

// Avoid direct access to the file
if (!defined('_PS_VERSION_'))
	exit;

/**
 * Class DEPOCarrier
 */
class DEPOCarrier extends CarrierModule
{

	public $id_carrier;

	/** @var  \DEPOClient\Client */
	protected $client = NULL;

	/**
	 * Hooks to be installed
	 * @var array
	 */
	protected $_hooks = [
		'updateCarrier',
		'extraCarrier',
		'header',

		'actionValidateOrder',
		'actionCarrierProcess',
		'actionOrderStatusUpdate',

		'displayAdminOrder',

		'displayOrderConfirmation',

		'displayOrderDetail'
	];

	public $extra_mail_vars = [];

	/*
	 * * Construct Method
	 * *
	 */

	public function __construct()
	{
		$this->name = 'depocarrier';
		$this->tab = 'shipping_logistics';
		$this->version = '0.2.0';
		$this->author = 'Webit';
		$this->bootstrap = true;

		parent::__construct();


		$this->displayName = $this->l('DEPO Carrier');
		$this->description = $this->l('Integrace dopravce DEPO.sk');
		$this->ps_versions_compliancy = ['min' => '1.5', 'max' => '1.5.99.99'];

		if (self::isInstalled($this->name)) {
			// Getting carrier list
			global $cookie;
			$carriers = Carrier::getCarriers($cookie->id_lang, true, false, false, NULL, PS_CARRIERS_AND_CARRIER_MODULES_NEED_RANGE);

			// Saving id carrier list
			$id_carrier_list = [];
			foreach ($carriers as $carrier)
				$id_carrier_list[] .= $carrier['id_carrier'];

			// Testing if Carrier Id exists
			$warning = [];
			if (!in_array((int) (Configuration::get('DEPO_CARRIER_ID')), $id_carrier_list))
				$warning[] = $this->l('DEPO_CARRIER_ID') . ' ';
			foreach($this->_getFormFields() as $f) {
				if (!Configuration::get($f)) {
					$warning[] = $this->l($f);
				}
			}
			if (!empty($warning)) {
				$this->warning = implode(' , ', $warning) . ' ' . $this->l('must be configured to use this module correctly') . ' ';
			}
		}
	}

	/**
	 * Installs module
	 * @return bool
	 */
	public function install()
	{
		$carrierConfig = [
			'name' => 'DEPO Carrier',
			'id_tax_rules_group' => 0,
			'active' => true,
			'deleted' => 0,
			'shipping_handling' => false,
			'range_behavior' => 0,
			'delay' => array('fr' => 'Description 1', 'en' => 'Description 1', Language::getIsoById(Configuration::get('PS_LANG_DEFAULT')) => 'Description 1'),
			'id_zone' => 1,
			'is_module' => true,
			'shipping_external' => true,
			'external_module_name' => $this->name,
			'need_range' => true
		];

		$id_carrier = $this->installExternalCarrier($carrierConfig);
		Configuration::updateValue('DEPO_CARRIER_ID', (int) $id_carrier);
		if (!parent::install() ||
				!$this->_installDb() ||
				!Configuration::updateValue('DEPO_OVERCOST', '') ||
				!$this->_installHooks() ||
				!$this->_installTab() ||
				!$this->_installOrderStates())
			return false;
		return true;
	}

	/**
	 * Installs database.
	 * @return bool
	 */
	protected function _installDb() {
		$files = ['cart_depo_ext.sql', 'order_depo_ext.sql', 'order_depo_ext_history.sql'];
		$db = Db::getInstance();
		foreach($files as $f) {
			$sql = file_get_contents(__DIR__.'/database/'.$f);
			$sql = str_replace('$DB_PREFIX$', _DB_PREFIX_, $sql);
			$db->execute($sql);
		}
		return TRUE;
	}

	/**
	 * Removes tables from database.
	 * @return bool
	 */
	protected function _uninstallDb() {
		$tables = ['cart_depo_ext', 'order_depo_ext', 'order_depo_ext_history'];
		foreach($tables as $t) {
			Db::getInstance()->execute('DROP TABLE '._DB_PREFIX_.$t);
		}
		return TRUE;
	}

	/**
	 * Returns used hooks.
	 * @return array
	 */
	protected function _getHookNames() {
		return $this->_hooks;
	}

	/**
	 * Installs hooks
	 * @return bool
	 */
	protected function _installHooks() {
		$result = true;
		foreach($this->_getHookNames() as $h) {
			if (!$this->registerHook($h)) {
				$result = false;
			}
		}
		return $result;
	}

	/**
	 * Uninstalls hooks
	 * @return bool
	 */
	protected function _uninstallHooks() {
		$result = true;
		foreach($this->_getHookNames() as $h) {
			if (!$this->unregisterHook($h)) {
				$result = false;
			}
		}
		return $result;
	}

	/**
	 * Resets hooks (usefull for development)
	 */
	protected function _resetHooks() {
		$this->_uninstallHooks();
		$this->_installHooks();
	}

	/**
	 * Installs tab.
	 * (We need this to have a functioning controller)
	 * @return mixed
	 */
	protected function _installTab()
	{
		$tab = new Tab();
		$tab->active = 1;
		$tab->name = array();
		$tab->class_name = 'AdminDepoCarrier';

		foreach (Language::getLanguages(true) as $lang) {
			$tab->name[$lang['id_lang']] = 'DEPO Carrier';
		}

		$tab->id_parent = -1;
		$tab->module = $this->name;

		return $tab->add();
	}

	/**
	 * Unistalls tab
	 * @return bool
	 */
	protected function _uninstallTab()
	{
		$id_tab = (int)Tab::getIdFromClassName('AdminDepoCarrier');

		if ($id_tab) {
			$tab = new Tab($id_tab);
			return $tab->delete();
		}

		return false;
	}

	/**
	 * Uninstalls module.
	 * Keeps database.
	 * @return bool
	 */
	public function uninstall()
	{
		// Uninstall
		if (!parent::uninstall() ||
				!$this->_uninstallDb() ||
				!$this->_uninstallHooks() ||
				!$this->_uninstallTab() ||
				!$this->_uninstallOrderStates() )
			return false;

		// Delete External Carrier
		$Carrier = new Carrier((int) (Configuration::get('DEPO_CARRIER_ID')));

		// If external carrier is default set other one as default
		if (Configuration::get('PS_CARRIER_DEFAULT') == (int) ($Carrier->id)) {
			global $cookie;
			$carriersD = Carrier::getCarriers($cookie->id_lang, true, false, false, NULL, PS_CARRIERS_AND_CARRIER_MODULES_NEED_RANGE);
			foreach ($carriersD as $carrierD)
				if ($carrierD['active'] AND ! $carrierD['deleted'] AND ( $carrierD['name'] != $this->_config['name']))
					Configuration::updateValue('PS_CARRIER_DEFAULT', $carrierD['id_carrier']);
		}

		// Then delete Carrier
		$Carrier->deleted = 1;
		if (!$Carrier->update())
			return false;

		// remove configurations
		$configs = [
			'OVERCOST',
			'USERNAME',
			'PASSWORD',
			'URL',
			'GOOGLE_MAP_API_KEY',
			'COD_MODULES',
			'CARRIER_ID',
			'PS_OS_DEPO_SHIPPED'
		];
		foreach($configs as $c) {
			Configuration::deleteByName('DEPO_'.$c);
		}

		return true;
	}

	/**
	 * Installs carrier.
	 * @param $config
	 * @return bool|int
	 * @throws PrestaShopDatabaseException
	 */
	public static function installExternalCarrier($config)
	{
		$carrier = new Carrier();
		$carrier->name = $config['name'];
		$carrier->id_tax_rules_group = $config['id_tax_rules_group'];
		$carrier->id_zone = $config['id_zone'];
		$carrier->active = $config['active'];
		$carrier->deleted = $config['deleted'];
		$carrier->delay = $config['delay'];
		$carrier->shipping_handling = $config['shipping_handling'];
		$carrier->range_behavior = $config['range_behavior'];
		$carrier->is_module = $config['is_module'];
		$carrier->shipping_external = $config['shipping_external'];
		$carrier->external_module_name = $config['external_module_name'];
		$carrier->need_range = $config['need_range'];

		$languages = Language::getLanguages(true);
		foreach ($languages as $language) {
			if ($language['iso_code'] == 'fr')
				$carrier->delay[(int) $language['id_lang']] = $config['delay'][$language['iso_code']];
			if ($language['iso_code'] == 'en')
				$carrier->delay[(int) $language['id_lang']] = $config['delay'][$language['iso_code']];
			if ($language['iso_code'] == Language::getIsoById(Configuration::get('PS_LANG_DEFAULT')))
				$carrier->delay[(int) $language['id_lang']] = $config['delay'][$language['iso_code']];
		}

		if ($carrier->add()) {
			$groups = Group::getGroups(true);
			foreach ($groups as $group)
				Db::getInstance()->autoExecute(_DB_PREFIX_ . 'carrier_group', array('id_carrier' => (int) ($carrier->id), 'id_group' => (int) ($group['id_group'])), 'INSERT');

			$rangePrice = new RangePrice();
			$rangePrice->id_carrier = $carrier->id;
			$rangePrice->delimiter1 = '0';
			$rangePrice->delimiter2 = '10000';
			$rangePrice->add();

			$rangeWeight = new RangeWeight();
			$rangeWeight->id_carrier = $carrier->id;
			$rangeWeight->delimiter1 = '0';
			$rangeWeight->delimiter2 = '10000';
			$rangeWeight->add();

			$zones = Zone::getZones(true);
			foreach ($zones as $zone) {
				Db::getInstance()->autoExecute(_DB_PREFIX_ . 'carrier_zone', array('id_carrier' => (int) ($carrier->id), 'id_zone' => (int) ($zone['id_zone'])), 'INSERT');
				Db::getInstance()->autoExecuteWithNullValues(_DB_PREFIX_ . 'delivery', array('id_carrier' => (int) ($carrier->id), 'id_range_price' => (int) ($rangePrice->id), 'id_range_weight' => NULL, 'id_zone' => (int) ($zone['id_zone']), 'price' => '0'), 'INSERT');
				Db::getInstance()->autoExecuteWithNullValues(_DB_PREFIX_ . 'delivery', array('id_carrier' => (int) ($carrier->id), 'id_range_price' => NULL, 'id_range_weight' => (int) ($rangeWeight->id), 'id_zone' => (int) ($zone['id_zone']), 'price' => '0'), 'INSERT');
			}

			// Copy Logo
			if (!copy(dirname(__FILE__) . '/carrier.jpg', _PS_SHIP_IMG_DIR_ . '/' . (int) $carrier->id . '.jpg'))
				return false;

			// Return ID Carrier
			return (int) ($carrier->id);
		}

		return false;
	}

	public function _installOrderStates() {
		/** @var OrderStateCore $os */
		$os = new OrderState();
		// lang field
		$os->name = [];

		$os->module_name = $this->name;
		$os->invoice = true;
		$os->color = '#8A2BE2';
		$os->logable = true;
		$os->delivery = true;
		$os->shipped = true;
		$os->paid = true;
		$os->pdf_invoice = false;
		$os->pdf_delivery = false;
		$os->unremovable = false;
		// mail options
		$os->send_email = true;
		// lang field
		$os->template = [];

		foreach(LanguageCore::getLanguages() as $l) {
			$os->name[$l['id_lang']] = 'Shipped (DEPO.sk)';
			$os->template[$l['id_lang']] = 'depo_shipped';

			// copy mail files (only those which exist)
			$module_path = dirname(__FILE__).'/views/templates/mails/'.$l['iso_code'].'/';
			$application_path = dirname(__FILE__).'/../../mails/'.$l['iso_code'].'/';

			$files = ['depo_shipped.txt', 'depo_shipped.html'];
			foreach($files as $f) {
				if (file_exists($module_path.$f)) {
					if (!copy($module_path.$f, $application_path.$f)) {
						return false;
					}
				}
			}
		}

		if ($os->add())	{
			// We save the order State ID in Configuration database
			Configuration::updateValue('PS_OS_DEPO_SHIPPED', $os->id);
		} else {
			return FALSE;
		}

		return TRUE;
	}

	/**
	 * Removes order states
	 * @return bool
	 * @throws PrestaShopException
	 */
	protected function _uninstallOrderStates() {
		$configs = ['PS_OS_DEPO_SHIPPED'];
		$os_ids = [];
		foreach($configs as $c) {
			$os_ids[] = Configuration::get($c);
		}
		$result = TRUE;
		foreach($os_ids as $id) {
			/** @var OrderStateCore $os */
			$os = new OrderState($id);
			$os->deleted = 1;
			if (!$os->update()) {
				$result = FALSE;
			}
		}
		foreach($configs as $c) {
			Configuration::deleteByName($c);
		}
		return $result;
	}

	/**
	 * Displays configuration page
	 * @return string
	 */
	public function getContent() {
		$output = '';
		if (Tools::isSubmit('depocarrier_form')) {
			$errors = $this->_postValidation();
			if (empty($errors)) {
				$this->_postProcess();
			} else {
				foreach($errors as $e) {
					$output .= '<div class="alert alert-warning">'
						.Tools::displayError($e)
						.'</div>';
				}
			}
		}
		$this->context->smarty->assign([
			// this default key should be changed to the one used by
			'default_depo_google_map_api_key' =>
				'AIzaSyDs9TaHiPhWTHaWRsbd3t1H-24EpLtKxNo'
		]);
		return $output .$this->_renderForm()
			. $this->display(__FILE__, 'config.tpl');
	}

	/**
	 * Configuration which can be changed from backoffice.
	 * @return array
	 */
	protected function _getFormFields() {
		$currencies = Currency::getCurrencies();
		$currency_fields = [];
		foreach($currencies as $currency) {
			$currency_fields[] = 'DEPO_COST_'.$currency['iso_code'];
		}

		return array_merge($currency_fields, [
			'DEPO_USERNAME',
			'DEPO_PASSWORD',
			'DEPO_URL',
			'DEPO_GOOGLE_MAP_API_KEY'
		]);
	}

	/**
	 * Returns modules which are currently created as COD.
	 * Either as array (list) or as values for usage in HelperForm.
	 * @param string $format 'list' or 'form'
	 * @return array
	 */
	protected function _getActiveCODModules($format = 'list') {
		$items = explode(',', Configuration::get('DEPO_COD_MODULES', '') );
		$ret = [];
		if ($format == 'list') {
			foreach($items as $i) {
				$ret[] = $i;
			}
		} elseif($format == 'form') {
			foreach($items as $i) {
				$ret['DEPO_COD_MODULES_' . $i] = TRUE;
			}
		}
		return $ret;
	}

	/**
	 * Updates active COD modules (sent from form)
	 */
	protected function _processActiveCODModules() {
		// set cashondelivery payments
		$modules = [];
		foreach($this->_getCODModules('list') as $m) {
			if (Tools::getValue('DEPO_COD_MODULES_'.$m) == 'on') {
				$modules[] = $m;
			}
		}
		Configuration::updateValue('DEPO_COD_MODULES', implode(',', $modules));
	}

	/**
	 * @param string $format list or form
	 * @return array
	 */
	protected function _getCODModules($format = 'form') {
		$mods = PaymentModule::getInstalledPaymentModules();
		$ret = [];
		foreach($mods as $m) {
			if ($format == 'form') {
				$ret[] = [
					'value' => $m['name'],
					'name' => $m['name'],
				];
			} elseif($format == 'list') {
				$ret[] = $m['name'];
			}
		}
		return $ret;
	}

	/**
	 * Renders configuration form.
	 * @return string
	 */
	protected function _renderForm()
	{
		$fields = $this->_getFormFields();
		$inputs = [];
		$fields_value = [];
		foreach($fields as $c) {
			$inputs[] = [
				'name' => $c,
				'label' => $this->l($c),
				'type' => 'text'
			];
			$fields_value[$c] =
				Tools::getValue($c, Configuration::get($c));
		}

		// input for cod modules
		$inputs[] = [
			'name' => 'DEPO_COD_MODULES',
			'label' => $this->l('DEPO_COD_MODULES'),
			'type' => 'checkbox',
			'class' => 't',
			'values' => [
				'query' => $this->_getCODModules('form'),
				'id' => 'value',
				'name' => 'name'
			]
		];
		$fields_value += $this->_getActiveCODModules('form');

		//
		$inputs[] = [
			'name' => 'DEV_DEPO_RESET_HOOKS',
			'label' => $this->l('DEV_DEPO_RESET_HOOKS'),
			'type' => 'radio',
			'class' => 't',
			'is_bool' => true,
			'values' => [
				['id' => 'reset_hooks_on', 'value' => 1, 'label' => $this->l('Yes')],
				['id' => 'reset_hooks_off', 'value' => 0, 'label' => $this->l('No')]
			]
		];
		$fields_value['DEV_DEPO_RESET_HOOKS'] = 0;

		$fields_form = [
			'form' => [
				'legend' => [
					'title' => $this->l('DEPO Carrier configuration'),
					'icon' => 'icon-wrench'
				],
				'input' => $inputs,
				'submit' => ['title' => $this->l('Save')]
			]
		];

		/** @var HelperFormCore $helper */
		$helper = new HelperForm();
		$helper->table = 'depocarrier';
		$helper->default_form_language = (int)Configuration::get('PS_LANG_DEFAULT');
		$helper->allow_employee_form_lang = (int)Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG');
		$helper->currentIndex = Tools::htmlentitiesUTF8($_SERVER['REQUEST_URI']);
		$helper->submit_action = 'depocarrier_form';
		$helper->token = Tools::getAdminTokenLite('AdminModules');
		$helper->tpl_vars = array(
			'fields_value' => $fields_value,
			'languages' => $this->context->controller->getLanguages()
		);

		return $helper->generateForm([$fields_form]);
	}

	/**
	 * Validates configuration form.
	 * @return array
	 */
	protected function _postValidation()
	{
		$errors = [];
		foreach($this->_getFormFields() as $f) {
			if (empty(Tools::getValue($f))) {
				$errors[] = $this->l('You have to set ') . $this->l($f);
			}
		}
		return $errors;
	}

	/**
	 * Applies changes from configuration form.
	 */
	protected function _postProcess()
	{
		foreach($this->_getFormFields() as $f) {
			Configuration::updateValue($f, Tools::getValue($f));
		}

		// update places cache
		$this->getClient()->updatePlacesCache();

		$this->_processActiveCODModules();

		// reset hooks
		if (Tools::getValue('DEV_DEPO_RESET_HOOKS', false) != false) {
			$this->_resetHooks();
		}

	}

	/**
	 * Adds css and js to header. (FRONT)
	 * @param $params
	 */
	public function hookHeader($params)
	{
		$cond = isset($this->context->controller->php_self) &&
				$this->context->controller->php_self == 'order';
		if ($cond) {
			$this->smarty->assign([
				'depo_google_map_api_key' => Configuration::get('DEPO_GOOGLE_MAP_API_KEY'),
				'DEPO_base_ajax_url' => $this->context->link->getModuleLink($this->name, 'ajax')
			]);

			return $this->display(__FILE__, 'header.tpl');
		}
	}

	/**
	 * After order validation, creates OrderDepoExt
	 * @param $params
	 * @throws PrestaShopException
	 */
	public function hookActionValidateOrder($params)
	{
		/** @var CartCore $cart */
		$cart = $params['cart'];
		/** @var OrderCore $order */
		$order = $params['order'];

		if ($cart->id_carrier == Configuration::get('DEPO_CARRIER_ID')) {
			$cart_depo = new CartDepoExt($cart->id);
			if (!Validate::isLoadedObject($cart_depo)) {
				throw new PrestaShopException('Missing CartDepoExt.');
			}

			// create order depo ext from cart depo ext
			$order_depo = new OrderDepoExt();
			$order_depo->id = $order->id;
			$order_depo->id_place = $cart_depo->id_place;
			$order_depo->number = NULL;
			$_POST['forceIDs'] = true;
			$order_depo->add(true, true);
			unset($_POST['forceIDs']);

		} else {
			return;
		}
	}

	/**
	 * Hook update carrier
	 * If new carrier is updated, we need to keep track of the id
	 * @param $params
	 */
	public function hookupdateCarrier($params)
	{
		if ((int) ($params['id_carrier']) == (int) (Configuration::get('DEPO_CARRIER_ID')))
			Configuration::updateValue('DEPO_CARRIER_ID', (int) ($params['carrier']->id));
	}

	/**
	 * Renders depo.tpl. Everytime the carrier is selected.
	 * @param $params
	 * @return mixed
	 */
	public function hookExtraCarrier($params)
	{
		if (Configuration::get('DEPO_CARRIER_ID') == $params['cart']->id_carrier) {
			$cart_depo_ext = new CartDepoExt($params['cart']->id);
			// if $cart_depo_ext is not in database
			if (!Validate::isLoadedObject($cart_depo_ext)) {
				$cart_depo_ext = NULL;
			} else {
				// delete binding if place with this id does not exist
				try {
					$this->getClient()->getPlace($cart_depo_ext->id_place);
				} catch (\DEPOClient\Exceptions\DEPOException $e) {
					if ($cart_depo_ext) {
						$cart_depo_ext->delete();
						$cart_depo_ext = NULL;
					}
				}
			}
			$this->smarty->assign([
				'locations' => $this->getClient()->getPlaces(),
				'cart_depo_ext' => $cart_depo_ext,
			]);
			return $this->display(__FILE__, 'depo.tpl');
		} else {
			return '';
		}
	}

	public function hookDisplayOrderConfirmation($params) {
		/** @var OrderCore $order */
		$order = $params['objOrder'];
		$order_depo = new OrderDepoExt($order->id);
		if (Validate::isLoadedObject($order_depo)) {
			$this->context->smarty->assign([
				'depo_place' => $this->getClient()->getPlace($order_depo->id_place)
			]);
			return $this->display(__FILE__, 'orderConfirmation.tpl');
		} else {
			return '';
		}
	}

	public function hookDisplayOrderDetail($params) {
		/** @var OrderCore $order */
		$order = $params['order'];
		$order_depo = new OrderDepoExt($order->id);
		if (Validate::isLoadedObject($order_depo)) {
			$this->context->smarty->assign([
				'order_depo' => $order_depo,
				'depo_place' => $this->getClient()->getPlace($order_depo->id_place)
			]);
			return $this->display(__FILE__, 'orderDetail.tpl');
		} else {
			return '';
		}
	}

	/**
	 * Returns client with set configuration.
	 * @return \DEPOClient\Client
	 */
	public function getClient() {
		try {
			if ($this->client === NULL) {
				$config = [
					// those directories have to be writable
					// by default those are in depo client directory
					'temp_dir' => __DIR__ . '/temp',
					'log_dir' => __DIR__ . '/log',
					'pdf_dir' => __DIR__ . '/pdf',
					// \Monolog\Logger costant
					// default value: Logger::ERROR
					'log_level' => Monolog\Logger::ERROR,
					// credentials for DEPO admin
					'username' => Configuration::get('DEPO_USERNAME'),
					'password' => Configuration::get('DEPO_PASSWORD'),
					// without trailing slash
					'base_url' => Configuration::get('DEPO_URL'),
					// refresh cache after
					'cache_ttl' => 3600,
				];

				$this->client = new \DEPOClient\Client($config);
			}

			return $this->client;

		} catch (\DEPOClient\Exceptions\DEPOException $e) {
			die('DEPO - Missing configuration');
		}
	}

	/**
	 * Front Methods
	 *
	 * If you set need_range at true when you created your carrier (in install method), the method called by the cart will be getOrderShippingCost
	 * If not, the method called will be getOrderShippingCostExternal
	 *
	 * $params var contains the cart, the customer, the address
	 * $shipping_cost var contains the price calculated by the range in carrier tab
	 *
	 */
	public function getOrderShippingCost($cart, $shipping_cost)
	{
		// This example returns shipping cost with overcost set in the back-office, but you can call a webservice or calculate what you want before returning the final value to the Cart
		if ($this->id_carrier == (int) (Configuration::get('DEPO_CARRIER_ID'))) {
			$currency = new Currency($cart->id_currency);
			return (float) Configuration::get('DEPO_COST_'.$currency->iso_code);
		}

		// If the carrier is not known, you can return false, the carrier won't appear in the order process
		return false;
	}

	/**
	 * See above.
	 * @param $cart
	 * @return bool|float
	 */
	public function getOrderShippingCostExternal($cart)
	{
		// This example returns shipping cost with overcost set in the back-office, but you can call a webservice or calculate what you want before returning the final value to the Cart
		if ($this->id_carrier == (int) (Configuration::get('DEPO_CARRIER_ID'))) {
			$currency = new Currency($cart->id_currency);
			return (float) Configuration::get('DEPO_COST_'.$currency->iso_code);
		}

		// If the carrier is not known, you can return false, the carrier won't appear in the order process
		return false;
	}

	public function hookDisplayAdminOrder($params) {
		$order_depo = new OrderDepoExt($params['id_order']);
		if ($order_depo->id > 0) {
			$this->context->smarty->assign([
				'rendered_snippets' => AdminDepoCarrierController::renderSnippets($order_depo)
			]);
			return $this->display(__FILE__, 'adminOrder.tpl');
		}
	}


	/**
	 * Updates carrier after order-carrier step.
	 * @param $params
	 * @throws PrestaShopException
	 */
	public function hookActionCarrierProcess($params) {
		/** @var CartCore $cart */
		$cart = $params['cart'];
		if ($cart->id_carrier == Configuration::get('DEPO_CARRIER_ID')) {
			$place_id = Tools::getValue('depo_place_id', false);
			// create depo cart
			$depo_cart = new CartDepoExt($cart->id);
			// set id place
			$depo_cart->id_place = $place_id;
			if (!Validate::isLoadedObject($depo_cart)) {
				$depo_cart->id = $cart->id;
				$_POST['forceIDs'] = true;
				$depo_cart->add();
				unset($_POST['forceIDs']);
			} else {
				$depo_cart->update();
			}
		}
	}

	/**
	 * Sets extra mail vars.
	 * @param $params
	 */
	public function hookActionOrderStatusUpdate($params) {
		/** @var OrderStateCore $new_os */
		$new_os = $params['newOrderStatus'];
		$id_order = $params['id_order'];
		/** @var OrderCore $order */
		$order = new Order($id_order);

		$order_depo = new OrderDepoExt($id_order);

		// if order is shipped and carrier is depo, add extra mail vars
		if ($new_os->shipped == 1 && $order->id_carrier == Configuration::get('DEPO_CARRIER_ID')) {
			$place = $this->getClient()->getPlace($order_depo->id_place);
			$this->extra_mail_vars['{depo_id_place}'] = $place->id;
			$this->extra_mail_vars['{depo_place_name}'] = $place->name;
			$this->extra_mail_vars['{depo_place_street}'] = $place->street;
			$this->extra_mail_vars['{depo_place_zip}'] = $place->zip;

			$open_hours = [];
			foreach($place->open_hours as $oh) {
				if ($oh['is_open']) {
					$open_hours[] = $oh['type'].' '.
						$oh['open_time'].' - '.$oh['close_time'];
				}
			}
			$this->extra_mail_vars['{depo_place_open_hours}'] = implode(', ', $open_hours);
		}

		if ($new_os->id == Configuration::get('PS_OS_DEPO_SHIPPED')) {
			if (!Validate::isLoadedObject($order_depo)) {
				return;
			}

			if ($order_depo->number !== NULL && !$order_depo->cancelled) {
				return;
			}

			$this->sendPackage($order, $order_depo);
		}

	}

	/**
	 * Sends package via API
	 * @param OrderCore $order
	 * @param OrderDepoExt $order_depo
	 * @return bool
	 * @throws PrestaShopException
	 */
	public function sendPackage($order, \OrderDepoExt $order_depo) {
		/** @var CustomerCore $customer */
		$customer = new Customer($order->id_customer);
		/** @var AddressCore $address */
		$address = new Address($order->id_address_delivery);
		/** @var CurrencyCore $currency */
		$currency = new Currency($order->id_currency);
		
		$is_cod = in_array($order->module, $this->_getActiveCODModules('list'));


		// Construct DEPO Package
		$package = new \DEPOClient\Entities\Package();
		$package->target = $order_depo->id_place;
		$package->insurance = $order->total_paid;
		$package->recipient_name = $address->firstname . ' ' . $address->lastname;
		$package->recipient_phone = $address->phone_mobile ? $address->phone_mobile : $address->phone;
		$package->recipient_email = $customer->email;
		$package->insurance_currency = $currency->iso_code;
		$package->sender_reference = $order->id;

		if ($is_cod) {
			$package->cod = $order->total_paid;
			$package->cod_currency = $currency->iso_code;
		}

		try {
			$order_depo->logToHistory();
			$this->getClient()->sendPackage($package);
			$order_depo->number = $package->number;
			$order_depo->cancelled = 0;
			$order_depo->update();
			return TRUE;
		} catch(\DEPOClient\Exceptions\DEPOException $e) {
			return FALSE;
		}
	}

	/**
	 * Cancels package via API
	 * @param OrderDepoExt $order_depo
	 * @return bool
	 * @throws PrestaShopException
	 */
	public function cancelPackage(\OrderDepoExt $order_depo) {
		try {
			$order_depo->logToHistory();
			$this->getClient()->cancelPackage($order_depo->number);
			$order_depo->cancelled = TRUE;
			$order_depo->update();
			return TRUE;
		} catch (\DEPOClient\Exceptions\DEPOException $e) {
			return FALSE;
		}
	}

}
